#!/bin/bash

#
#  This is a small part of very big administration script
#  used in UCVT for manipulations with user profiles and directories
#  on the Linux/Domino server
#
#  Written by Evseev/EBCEEB at Jan 2002.
#  Contact evseev@ucvt.ru or visit http://www.evseev.ucvt.ru
#
#  Use switch "-d" for testing before actual running
#


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Variables
#
 COUNTRY_SUFFIX=ru
 DOMUSERS_ACCOUNT=domusers

SCRIPT=`basename $0`
# echo SCRIPT = ${SCRIPT}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  When "$1" is "-d", then output all commands instead of executing
#
if [ "$1" = "-d" ]; then
	shift
	[ -z "$doit" ] && export doit=echo
elif [ "$1" = "-dd" ]; then
	shift
	[ -z "$doit" ] && export doit=echo
	set -x
fi


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Error handling: report and exit
#

fail() {
	echo ${SCRIPT}: ERROR: $*, abort. 1>&2
	exit 1
}

warning() {
	echo ${SCRIPT}: Warning: $*. 1>&2
}

# $1 = min args count, $2 = actual args count, $3 = (optional) max args count
checkArgsCount()
{
	MINARGS=$1
	 ACTUAL=$2
	MAXARGS=$3
	[ $# -lt 3 ] && MAXARGS=$MINARGS
	[ $# -lt 2 -o $# -gt 3 ] && return 1
	[ $ACTUAL -lt $MINARGS ] && return 1
	[ $ACTUAL -gt $MAXARGS ] && return 1
	return 0
}

# $1 = caller name
# $2 = min args count, $3 = actual args count, $4 = (optional) max args count
failOnArgsCount() {
	checkArgsCount $2 $3 $4 || fail $1: args count = $3, must be $2...$4
}


# $1 = caller name, $2 = usage syntax, $3 = example
# $4 = min args count, $5 = actual args count, $6 = max args count
usage() {
	failOnArgsCount Usage 5 $# 6
	checkArgsCount $4 $5 $6 && return 0
	echo ___Usage: $SCRIPT $1 $2
	echo _Example: $SCRIPT $1 $3
	exit 1
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  hostname2dirname
#
#  Build directory name where to store files appropriated to given host.
#
#     Input: $1 = host name (FQDN)
#    Output: $DIRECTORY_NAME = directory name
#   Example: notessrv.spb05.ucvt.ru => ucvt/spb05/notessrv
#   Skips final (country or organization) part of hostname using $CONTRY_SUFFIX
#
hostNameToDirectoryName() {
	[ -z "$1" ] && return
	DIRECTORY_NAME=
	for n in 9 8 7 6 5 4 3 2 1; do
		w=`echo $1 | cut -d . -f $n`
		[ "$w" != "" -a "$w" != "${COUNTRY_SUFFIX}" ] && DIRECTORY_NAME=${DIRECTORY_NAME}/$w
	done
}

localHostToDirectoryName() {
        if [ "$SAVED_HOSTDIR" = "" ]; then
		hostNameToDirectoryName `hostname`
		SAVED_HOSTDIR=$DIRECTORY_NAME
	else
		DIRECTORY_NAME=$SAVED_HOSTDIR
	fi
}

#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
# GetDirectoryName:
#          $1 = group name, $2 = user login => $DIRECTORY_NAME = full home path
# Example A: $1 = teachers, $2 = evseev     => $DIRECTORY_NAME = /var/ucvt/spb05/notessrv/teachers/evseev
# Example B: $1 = students, $2 = n2199/s14  => $DIRECTORY_NAME = /var/ucvt/spb05/notessrv/students/n2199/s14
# Example C: $1 = finance,  $2 = ""         => $DIRECTORY_NAME = /var/ucvt/spb05/notessrv/finance
#
getDirectoryName() {
	localHostToDirectoryName
	DIRECTORY_NAME=/var${DIRECTORY_NAME}
	[ "$1" != "" ] && DIRECTORY_NAME=${DIRECTORY_NAME}/$1
	[ "$2" != "" ] && DIRECTORY_NAME=${DIRECTORY_NAME}/$2
	[ -d ${DIRECTORY_NAME} ] && return 0 || return 1
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
#  Check existing users and groups
#
# $1 = filename, $2 = account name => return $?
isAccountPresent() {
	grep -e "^${2}:" $1 > /dev/null && return 0 || return 1
}

isUnixGroupRegistered() {
	failOnArgsCount IsUnixGroupRegistered 1 $#
	isAccountPresent /etc/group $1
}

isUnixUserRegistered() {
	failOnArgsCount IsUnixUserRegistered 1 $#
	isAccountPresent /etc/passwd $1
}

isSambaUserRegistered() {
	failOnArgsCount IsSambaUserRegistered 1 $#
	isAccountPresent /etc/samba/smbpasswd $1
}

# $1 = directory where to create lock file
preventDeletion() {
	failOnArgsCount preventDeletion 1 $#
	$doit touch            $1/.lock && \
	$doit chown root:wheel $1/.lock && \
	$doit chmod 400        $1/.lock
}

isNotesUserDirectoryExist() {
	failOnArgsCount NotesUserDirectoryExist 1 $#
	getDirectoryName $DOMUSERS_ACCOUNT $1
}


#**********************************************************************
#
#    Notes users
#

# $1 = username
registerNotesUser()
{
        usage RegisterNotesUser username ivanov 1 $#
	isUnixUserRegistered $1 || \
		fail User $1 must be already registered in Unix
	isSambaUserRegistered $1 || \
		fail User $1 must be already registered in Microsoft Network
        isNotesUserDirectoryExist $1 && \
		warning User already have directory for Lotus Notes files || \
		$doit mkdir -m u=rwx,g=srwx $DIRECTORY_NAME
	$doit usermod -G `id -Gn $1 | tr " " ,`,$DOMUSERS_ACCOUNT $1	
	$doit chown notes:$1 $DIRECTORY_NAME
	preventDeletion      $DIRECTORY_NAME
	echo Dont forget place appropriate $DIRECTORY_NAME/USER.ID!
}

# $1 = username
resetNotesPerms() {
        usage ResetNotesPerms username ivanov 1 $#
	isUnixUserRegistered $1 || \
		fail User $1 must be already registered in Unix
        isNotesUserDirectoryExist $1 || \
		fail User $1 dont have directory for Lotus Notes files

	# Add user to "domusers" group
	if ! id -Gn $1 | grep $DOMUSERS_ACCOUNT > /dev/null; then
		$doit usermod -G $(id -Gn $1 | tr " " ,),$DOMUSERS_ACCOUNT $1
	fi

	# Change ownership: owner = "notes", group = actual domino user
	$doit chown -R notes:$1 $DIRECTORY_NAME

	# Change permissions:
	# "notes" user = read/write/chdir,
	#  group (actually domino user) = read/write/chdir
	$doit chmod -R ug=rwX $DIRECTORY_NAME

	# Create copy of USER.ID to USER.ID0
	if [ -f $DIRECTORY_NAME/user.id ]; then
		if [ ! -f $DIRECTORY_NAME/user.id0 ]; then
			$doit cp $DIRECTORY_NAME/user.id $DIRECTORY_NAME/user.id0
			$doit chown notes:$1 $DIRECTORY_NAME/user.id0
		fi
		$doit chmod 040 $DIRECTORY_NAME/user.id0
	fi
	
	preventDeletion $DIRECTORY_NAME
}

# $1 = username
wheelNotesPerms() {
        usage OpenNotesPerms username ivanov 1 $#
        isNotesUserDirectoryExist $1 || \
		fail User $1 dont have directory for Lotus Notes files
	chown -R notes:wheel $DIRECTORY_NAME
	chmod -R ug=rwX      $DIRECTORY_NAME
}


#~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

resetNotesPerms $*
exit $?

cmd=$1
shift

case $cmd in
	-lib* | lib*)
		dontExit=1 ;;
	addnotes | notesadd)           registerNotesUser $* ;;
	notesperms)			resetNotesPerms $* ;;
	noteswheel)			wheelNotesPerms $* ;;

	*)
		echo Usage: $SCRIPT prepare, adduser, resetuser, deluser, ...
		exit 1
esac

[ -z "$dontExit" ] && exit $?

## EOF ##
